import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;
import java.io.*;
import javax.swing.*;
import java.awt.BorderLayout;
import java.awt.Event;
import java.awt.event.*;
import java.awt.FileDialog;
import java.text.MessageFormat;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;

public class ValFront extends JPanel {
	JTabbedPane tabbedPane = new JTabbedPane();
	protected Hashtable entities = new Hashtable();
	public boolean isValid = false;
	UsageTab usageTab;
	ResourceBundle resource;
	static final String warmupDoc = "<?xml version='1.0'?><!DOCTYPE root [ <!ELEMENT root (child)*><!ELEMENT child (#PCDATA)><!ATTLIST child attr CDATA #REQUIRED>]><root><child attr='value'>text</child><child attr='value'>text</child><child attr='value'>text</child><child attr='value'>text</child><child attr='value'>text</child><child attr='value'>text</child><child attr='value'>text</child><child attr='value'>text</child><child attr='value'>text</child><child attr='value'>text</child></root>";

	public ValFront() {
		setLayout(new BorderLayout());
		add("Center", tabbedPane);
		resource = ResourceBundle.getBundle("ValFrontResource");
		addUsageTab();
	}

	public ResourceBundle getResource() { return resource; }
	
	/** warmup parser and validator: do static initialization and
        cause some optimization. */
	public void warmupParser() {
		try {
			SAXParserFactory factory = SAXParserFactory.newInstance();
			factory.setValidating(true);
			SAXParser parser = factory.newSAXParser();
			InputStream stream = new StringBufferInputStream(warmupDoc);
			parser.parse(stream, new DefaultHandler());
		}catch(Exception e){
			e.printStackTrace();
		}
	}
	/** clean up state, parse, report errors per entity. */
	public boolean loadFile(String filename) {
		tabbedPane.removeAll();
		entities.clear();
		//entities.put(filename, new ErrorsPerEntity(filename));
		SAXParserFactory factory = SAXParserFactory.newInstance();
		SAXParser parser;
		isValid = false;
		try {
			factory.setValidating(true);
			factory.setFeature("http://xml.org/sax/features/namespaces",false);
			try {
				factory.setFeature(
				   "http://apache.org/xml/features/continue-after-fatal-error",
				   true);
			}catch(SAXException se){
			}
			//factory.setNamespaceAware(false);
			parser = factory.newSAXParser();
			isValid = true;
			parser.parse(new File(filename), new HandlerBase() {
					public void error(SAXParseException e) {
						addErrorMessage("ERROR", e);
						isValid = false;
					}
					public void fatalError(SAXParseException e) {
						addErrorMessage("FATAL", e);
						isValid = false;
					}
					public void warning(SAXParseException e) {
						addErrorMessage("WARNING", e);
					}
				});
		} catch (Exception ex) {
			return false;
		}
		if(isValid) {
			ErrorsPerEntity errors = new ErrorsPerEntity(filename);
			errors.append("[OK] document is valid.");
		}
		addUsageTab();
		tabbedPane.requestFocus();
		return isValid;
	}

	protected void addErrorMessage(String type, SAXParseException e) {
		Object [] args = { new Integer(e.getLineNumber()),
						   new Integer(e.getColumnNumber()) };
		String errorStr = resource.getString(type)
			+ MessageFormat.format(resource.getString("LINE_AND_COLUMN"), args)
			+ e.getMessage() + "\n";
		String sid = e.getSystemId();
		ErrorsPerEntity errors = (ErrorsPerEntity)entities.get(sid);
		if(errors == null){
			errors = new ErrorsPerEntity(sid);
			entities.put(sid, errors);
		}
		errors.append(errorStr);
	}

	protected void addUsageTab() {
		if(usageTab == null) {
			usageTab = new UsageTab();
		}
		tabbedPane.addTab(usageTab.getTitle(), usageTab);
	}
	
	class ErrorsPerEntity {
		JTextArea errors;
		ErrorsPerEntity(String entityURL) {
			String title = entityURL.substring(entityURL.lastIndexOf('/')+1);
			errors = new JTextArea();
			errors.setEditable(false);
			errors.setLineWrap(true);
			errors.setWrapStyleWord(true);
			tabbedPane.addTab(title, 
							  new JScrollPane(errors,
								JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
								JScrollPane.HORIZONTAL_SCROLLBAR_NEVER));
		}
		void append(String msg) {
			errors.append(msg);
		}
	}

	class UsageTab extends JScrollPane {
		JTextArea area = new JTextArea();
		UsageTab() {
			super(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				  JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
			area.setEditable(false);
			area.setLineWrap(true);
			area.setWrapStyleWord(true);
			area.append(resource.getString("USAGE"));
			setViewportView(area);
		}
		String getTitle() {
			return resource.getString("T_USAGE");
		}
	}

	static ValFront app;
	static JFrame f;

	public static void main(String[] args) {
		app = new ValFront();
		if(args.length == 1) {
			app.loadFile(args[0]);
		}else{
			app.warmupParser();
		}
		f = new JFrame("XML Validator Frontend");
		// menu...
		JMenuBar bar = new JMenuBar();
		JMenu fileMenu = new JMenu(app.getResource().getString("M_FILE")
								   +"(F)");
		fileMenu.setMnemonic('F');
		JMenuItem openItem = 
			new JMenuItem(app.getResource().getString("M_OPEN")
						  +"(O)...");
		openItem.setMnemonic('O');
		openItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O,
													   Event.CTRL_MASK));
		fileMenu.add(openItem);
		bar.add(fileMenu);
		f.setJMenuBar(bar);
		// panel...
		f.getContentPane().add("Center",app);
		// listener...
		openItem.addActionListener(new ActionListener() {
				String dir = ".";
				FileDialog dlg = new FileDialog(f, "Open File",
												FileDialog.LOAD);
				public void actionPerformed(ActionEvent e) {
					dlg.setDirectory(dir);
					dlg.setFile("*.xml");
					dlg.setModal(true);
					dlg.show();
					dir = dlg.getDirectory();
					if(dlg.getFile() != null) {
						File file = new File(dir, dlg.getFile());
						if(file.exists()){
							f.setTitle("validating: " + dlg.getFile());
							if(! app.loadFile(file.getPath())){
								//java.awt.Toolkit.getDefaultToolkit().beep();
								f.setTitle("error: " + dlg.getFile());
							}else{
								f.setTitle("valid: " + dlg.getFile());
							}
						}else{
							java.awt.Toolkit.getDefaultToolkit().beep();
						}
					}
				}
			});
		f.addWindowListener(new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
					System.exit(0);
				}
				public void windowClosed(WindowEvent e) {
					System.exit(0);
				}
			});
		// frame...
		f.setSize(480,320);
		f.setVisible(true);
	}
}
